Option Explicit On 
Option Strict On
Option Compare Binary

Imports System
Imports System.Windows.Forms

' <remarks>
'   Stellt Funktionalitt zum Enumerieren aller 
'   Steuerelemente bestimmter Typen in einer Menge 
'   von Containern bereit, die nicht in einem der in 
'   einer weiteren Menge von Containern liegenden 
'   Container enthalten sind.
' </remarks>
Public Class ControlEnumerator

    ' <summary>
    '   Wird fr jedes whrend des Enumerationsvorgangs 
    '   gefundene Steuerelement ausgelst.
    ' <summary>
    ' <param name="ctrControl">Steuerelement, das gefunden wurde.</param>
    Public Event ControlFound(ByVal sender As Object, _
      ByVal e As ControlFoundEventArgs)

    ' Private Variablen zum Speichern der Eigenschaftswerte.
    Private m_actrExcludedContainers() As Control
    Private m_atypInterestingControlTypes() As Type
    Private m_ctrRootContainers() As Control

    ' <summary>
    '   Ruft fr jedes Steuerelement, das einem der 
    '   Steuerelemente in <c>RootContainers</c>
    '   untergeordnet ist, jedoch nicht <c>ExcludedContainers</c>, 
    '   und zustzlich noch einen
    '   der Datentypen in <c>InterestingControlTypes</c> aufweist,
    '   das Ereignis <c>ControlFound</c> auf.
    ' </summary>
    Public Sub Enumerate()
        Dim ctr As Control
        For Each ctr In Me.RootContainers
            DoEnumeration(ctr)
        Next ctr
    End Sub

    ' <summary>
    '   Fhrt eine rekursive Enumeration der interessanten
    '    Steuerelemente in einem Container durch.
    ' </summary>
    ' <param name="ctrControl">Steuerelement, fr 
    '    das die Enumeration aufgerufen wird.</param>
    Private Sub DoEnumeration( _
      ByVal ctrControl As Control)

        ' Der Container ist nicht einer der "verbotenen" Container.
        If Not IsExcludedContainer(ctrControl) Then

            ' Es handelt sich nicht um einen Wurzelcontainer (diese
            '  sind als "Ergebniskandidaten" ausgenommen).
            If Not IsRootContainer(ctrControl) Then
                If IsInterestingControl(ctrControl) Then
                    RaiseEvent ControlFound(Me, _
                      New ControlFoundEventArgs(ctrControl))
                End If
            End If

            ' Wenn der Container berhaupt Kinder besitzt.
            If ctrControl.HasChildren Then

                ' Fr alle Steuerelemente rekursiv aufrufen.
                Dim ctr As Control
                For Each ctr In ctrControl.Controls
                    DoEnumeration(ctr)
                Next ctr
            End If
        End If
    End Sub

    ' <summary>
    '   Gibt zurck, ob das angegeben Steuerelement in der 
    '   Sammlung der Wurzelcontainer enthalten ist.
    ' </summary>
    ' <param name="ctrControl">Zu untersuchendes Steuerelement.</param>
    ' <returns>
    '   Boolescher Wert, der angibt, ob das 
    '   im Parameter <paramref name="ctrControl"/>
    '   bergebene Steuerelement ein Wurzelcontainer ist.
    ' </returns>
    Private Function IsRootContainer( _
      ByVal ctrControl As Control) As Boolean
        Dim ctr As Control
        For Each ctr In Me.RootContainers
            If ctrControl Is ctr Then
                Return True
            End If
        Next ctr
        Return False
    End Function

    ' <summary>
    '   Gibt zurck, ob es sich beim im Parameter 
    '   <paramref name="ctrControl"/> bergebenen
    '   Steuerelement um einen von der Enumeration
    '   ausgenommenen Container handelt.
    ' </summary>
    ' <param name="ctrControl">Zu untersuchendes Steuerelement.</param>
    ' <returns>
    '   Boolescher Wert, der angibt, ob das im Parameter 
    ' <paramref name="ctrControl"/> bergebene 
    ' Steuerelement eine uninteressanter Container ist.
    ' </returns>
    Private Function IsExcludedContainer( _
      ByVal ctrControl As Control) As Boolean
        Dim ctr As Control
        For Each ctr In Me.ExcludedContainers
            If ctrControl Is ctr Then
                Return True
            End If
        Next ctr
        Return False
    End Function

    ' <summary>
    '   Gibt zurck, ob das im Parameter 
    '   <paramref name="ctrControl"/> angegebene Steuerelement
    '   Instanz eines interessanten Typs ist.
    ' </summary>
    ' <param name="ctrControl">Zu untersuchendes Steuerelement.</param>
    ' <returns>
    '   Boolescher Wert, der angibt, ob das im Parameter 
    '   <paramref name="ctrControl"/> bergebene Steuerelement interessant ist.
    ' </returns>
    Private Function IsInterestingControl( _
      ByVal ctrControl As Control) As Boolean
        Dim typ As Type
        For Each typ In Me.InterestingControlTypes
            If ctrControl.GetType() Is typ Then
                Return True
            End If
        Next typ
        Return False
    End Function

    ' <summary>
    '   Gibt eine Menge von Steuerelementcontainern, 
    '   in denen nicht weiter gesucht werden soll,
    '   an oder gibt sie zurck.
    ' </summary>
    ' <value>Menge von Containern, deren enthaltene 
    '   Steuerelemente nicht betrachtet werden.</value>
    Public Property ExcludedContainers() As Control()
        Get
            Return m_actrExcludedContainers
        End Get
        Set(ByVal Value() As Control)
            m_actrExcludedContainers = Value
        End Set
    End Property

    ' <summary>
    '   Gibt eine Menge interessanter Steuerelementtypen
    '   an oder gibt sie zurck. Steuerelemente
    '   eines dieser Typen, die whrend der Enumeration
    '   durchlaufen werden, werden ausgegeben.
    ' </summary>
    ' <vaule>Menge interessanter Steuerelementtypen.</value>
    Public Property InterestingControlTypes() As Type()
        Get
            Return m_atypInterestingControlTypes
        End Get
        Set(ByVal Value() As Type)
            m_atypInterestingControlTypes = Value
        End Set
    End Property

    ' <summary>
    '   Gibt eine Menge von Wurzelcontainern, die 
    '   auf untergeordnete Steuerelemente eines
    '   der Typen in <c>InterestingControlTypes</c> 
    '   untersucht werden soll, an oder gibt
    '   sie zurck.
    ' </summary>
    ' <value>Menge von zu durchsuchenden Wurzelcontainern.</value>
    Public Property RootContainers() As Control()
        Get
            Return m_ctrRootContainers
        End Get
        Set(ByVal Value() As Control)
            m_ctrRootContainers = Value
        End Set
    End Property
End Class